
const float indentWidth = 12.f * UI::GetScale();

abstract class ListItem {
    string name;
    string identifier;

    ListItem(const string &in name, const string &in identifier) {
        this.name = name; // might be "" while requesting
        this.identifier = identifier;
    }

    bool Render(const string &in index, int indent, bool allowclose = true) {
        /*UI::SetCursorPos(UI::GetCursorPos() + vec2(10.f * indent * UI::GetScale(),0));
          if (this.open) {
             UI::SetNextItemOpen(this.open);
        }*/
        return false;
    }
}

class ListList : ListItem {
    bool open;
    array<ListItem@> maplists;

    ListList(const string &in name, const string &in identifier, bool collapsed = false) {
        super(name, identifier);
        this.maplists = {};
        this.open = !collapsed;
    }
    ListList(const string &in name, const string &in identifier, array<ListItem@> maplists, bool collapsed = false) {
        super(name, identifier);
        this.maplists = maplists;
        this.open = !collapsed;
    }

    bool Render(const string &in index, int indent, bool allowclose = true) override {
        UI::SetCursorPos(UI::GetCursorPos() + vec2(indentWidth * indent,0));
        if (allowclose) {
            if (UI::Button(Icons::Times + '##:'+index)) {return true;}
        } else {
            UI::BeginDisabled();
            UI::Button(Icons::Times + '##:'+index);
            UI::EndDisabled();
        }
        UI::SameLine();
        if (this.open) {
            UI::SetNextItemOpen(this.open);
        }
        if (this.open = UI::CollapsingHeader(this.name + '##:'+index)) {
            if (this.maplists.Length > 0) {
                for (uint i = 0; i < this.maplists.Length; i++) {
                    if (this.maplists[i].Render(index + "." + tostring(i), indent+1)) {
                        this.maplists.RemoveAt(i);
                        i--;
                    }
                }
            }
        }
        return false;
    }

    // add a map list to the maps list list
    void InsertLast(ListItem@ maplist) {
        // for (uint i = 0; i < this.maplists.Length; i++) {
        //     if (this.maplists[i].identifier == maplist.identifier) {
        //         return;
        //     }
        // }
        this.maplists.InsertLast(maplist);
    }
    void InsertFirst(ListItem@ maplist) {
        // for (uint i = 0; i < this.maplists.Length; i++) {
        //     if (this.maplists[i].identifier == maplist.identifier) {
        //         return;
        //     }
        // }
        this.maplists.InsertAt(0, maplist);
    }
    ListItem@ opIndex (uint index) {
        return this.maplists[index];
    }
    void RemoveAt(uint index) {
        this.maplists.RemoveAt(index);
    }
    uint get_Length() const property {
        return this.maplists.Length;
    }
}

UI::Font@ boldFont = UI::LoadFont("DroidSans-Bold.ttf", 16);

class NewsList : ListItem {
    string header;
    string body;
    string url;
    bool hascontents = false;
    bool open;

    NewsList(const string &in name, const string &in identifier, const string &in url, bool collapsed = false) {
        super(name, identifier);
        this.url = url;
        this.hascontents = false;
        this.open = !collapsed;
    }

    void GetContents() {
        NadeoActivities::GetNewsFromNadeoNews(NadeoServices::BaseURLLive() + "/api/token/club/" + this.url, this);
    }

    bool Render(const string &in index, int indent, bool allowclose = true) override {
        if (this.open && !this.hascontents) {
            startnew(CoroutineFunc(this.GetContents));
            this.hascontents = true;
        }
        UI::SetCursorPos(UI::GetCursorPos() + vec2(indentWidth * indent,0));
        if (allowclose) {
            if (UI::Button(Icons::Times + '##:'+index)) {return true;}
        } else {
            UI::BeginDisabled();
            UI::Button(Icons::Times + '##:'+index);
            UI::EndDisabled();
        }
        UI::SameLine();
        if (this.open) {
            UI::SetNextItemOpen(this.open);
        }
        if (this.open = UI::CollapsingHeader(this.name + '##:'+index)) {
            UI::SetCursorPos(UI::GetCursorPos() + vec2(indentWidth * (indent+1),0));
            UI::PushFont(boldFont);
            UI::Text(this.header);
            UI::PopFont();
            UI::SetCursorPos(UI::GetCursorPos() + vec2(indentWidth * (indent+1),0));
            UI::TextWrapped(this.body);
        }
        return false;
    }
}

class OtherActivity : ListItem {
    bool open;

    OtherActivity(const string &in name, const string &in identifier, bool collapsed = false) {
        super(name, identifier);
        this.open = !collapsed;
    }

    bool Render(const string &in index, int indent, bool allowclose = true) override {
        UI::SetCursorPos(UI::GetCursorPos() + vec2(indentWidth * indent,0));
        if (allowclose) {
            if (UI::Button(Icons::Times + '##:'+index)) {return true;}
        } else {
            UI::BeginDisabled();
            UI::Button(Icons::Times + '##:'+index);
            UI::EndDisabled();
        }
        UI::SameLine();
        // if (this.open) {
        //     UI::SetNextItemOpen(this.open);
        // }
        // if (this.open = UI::CollapsingHeader(this.name + '##:'+index)) {
        //     //
        // }
        UI::Text(this.name);
        return false;
    }
}


