
class Map : ListItem {
    // identifier could be uid, id or tmx number
    string type = ""; // e.g. TM_Race or TM_Stunt or TM_Platform
    string url;

    string tmxnumber = "";
    string nadeouid = "";

    Map(const string &in name, const string &in identifier, const string &in type, const string &in url = "") {
        super(name, identifier);
        if (name == "") {
            print('Missing map name ' + identifier);
        }
        this.type = type;
        this.url = url;

        if (NadeoMaps::LooksLikeUid(identifier)) {
            this.nadeouid = identifier;
        } else if (TMXMaps::LooksLikeNumber(identifier)) {
            this.tmxnumber = identifier;
        } else if (TMXMaps::LooksLikeShortLink(identifier)) {
            this.tmxnumber = TMXMaps::GetNumberFromShortLink(identifier);
        } else if (TMXMaps::LooksLikeFullLink(identifier)) {
            this.tmxnumber = TMXMaps::GetNumberFromFullLink(identifier);
        } else if (TMXMaps::LooksLikeNewLink(identifier)) {
            this.tmxnumber = TMXMaps::GetNumberFromNewLink(identifier);
        } else if (TrackmaniaioMaps::LooksLikeMap(identifier)) {
            this.nadeouid = TrackmaniaioMaps::GetUidFromURL(identifier);
        } else if (FileMaps::LooksLikeFileMap(identifier)) {
            // this.nadeouid = FileMaps::GetUidFromFileMap(identifier);
            this.nadeouid = "";
        }
    }

    // must be called from yieldable; if can't yield manually check if map.nadeouid == ""
    string getuid() {
        if (this.nadeouid == "") {
            this.nadeouid = GetUid(this.identifier);
            if (this.nadeouid != "") {
                this.identifier = this.nadeouid; // easier to use in future without re-checking
            }
        }
        return this.nadeouid;
    }
    void gettmxid() {
        if (this.tmxnumber == "") {
            this.tmxnumber = GetTMXId(this.identifier);
            if (this.tmxnumber != "" && this.tmxnumber != '0') {
                this.identifier = this.tmxnumber; // easier to use in future without re-checking
            }
        }
    }

    // must be called from main (so can't activate anything else while working)
    void Play() {
        if (maptoload != "") {return;}
        if (this.url == "") {
            maptoload = this.identifier;
            loadmapas = LoadAs::Solo;
        } else {
            _PlayMapFromUrl(this.url, LoadAs::Solo, this.type, this.name);
        }
    }
    // must be called from main (so can't activate anything else while working)
    void Edit() {
        if (maptoload != "") {return;}
        if (this.url == "") {
            maptoload = this.identifier;
            loadmapas = LoadAs::Editor;
        } else {
            _PlayMapFromUrl(this.url, LoadAs::Editor, "", this.name);
        }
    }

    // must be called as startnew
    void Save() {
        if (this.url == "") {
            SaveMap(this.identifier);
        } else {_SaveMapFromUrl(this.url, Text::StripOpenplanetFormatCodes(this.name));}
    }

    // must be called as startnew
    void Favourite() {
        _FavouriteMapFromUid(this.getuid());
    }
    // must be called as startnew
    void Unfavourite() {
        _UnfavouriteMapFromUid(this.getuid());
    }

    // will return true if the maps match and false if they might not match (if one has tmx id and one nadeo id it might not)
    bool opEquals(Map@ other) {
        if (this.nadeouid != "" && other.nadeouid != "") {return this.nadeouid == other.nadeouid;}
        if (this.tmxnumber != "" && other.tmxnumber != "") {return this.tmxnumber == other.tmxnumber;}
        return false;
    }
    // checks if the map matches a nadeo uid / tmx id (if it doesn't have that id saved, will return false)
    bool opEquals(const string &in other) {
        if (this.nadeouid != "" && NadeoMaps::LooksLikeUid(other)) {return this.nadeouid == other;}
        if (this.tmxnumber != "" && TMXMaps::LooksLikeNumber(other)) {return this.tmxnumber == other;}
        return false;
    }

    bool Render(const string &in index, int indent, bool allowclose = true) override {
        UI::SetCursorPos(UI::GetCursorPos() + vec2(indentWidth * indent,0));
        if (isLoadingMap) {
            UI::BeginDisabled();
            UI::Button(this.name + "##:" + index + ":" + this.identifier);
            UI::EndDisabled();
        } else {
            if (UI::Button(this.name + "##:" + index + ":" + this.identifier)) {
                startnew(CoroutineFunc(this.Play));
            }
        }
        UI::SameLine();
        if (UI::Button(Icons::Download + "##:" + index + ":" + this.identifier)) {startnew(CoroutineFunc(this.Save));}
        UI::SameLine();
        if (isLoadingMap) {
            UI::BeginDisabled();
            UI::Button(Icons::Pencil + "##:" + index + ":" + this.identifier);
            UI::EndDisabled();
        } else {
            if (UI::Button(Icons::Pencil + "##:" + index + ":" + this.identifier)) {
                startnew(CoroutineFunc(this.Edit));
            }
        }
        UI::SameLine();
        if (!Favourites::isValid || Favourites::isRefreshing) {
            UI::BeginDisabled();
            UI::Button(Icons::Star + "##:" + index + ":" + this.identifier);
            UI::EndDisabled();
        } else {
            if (Favourites::Favourites.Contains(this)) {
                if (UI::Button(Icons::StarO + "##:" + index + ":" + this.identifier)) {startnew(CoroutineFunc(this.Unfavourite));}
                if (UI::IsItemHovered()) {
                    UI::BeginTooltip();
                    UI::Text('Remove favourite');
                    UI::EndTooltip();
                }
            } else {
                if (UI::Button(Icons::Star + "##:" + index + ":" + this.identifier)) {startnew(CoroutineFunc(this.Favourite));}
                if (UI::IsItemHovered()) {
                    UI::BeginTooltip();
                    UI::Text('Add favourite');
                    UI::EndTooltip();
                }
            }
        }
        return false;
    }
}