
enum MapListType {
    Favourites,
    
    Campaign,
    Room,
    MapUploads,
    SkinUploads,
    ItemUploads,
    News,
    MapReview,
    Competition,
    Rankings,
    Advertisement,
    Team,

    TMXMappack,

    None

}

class MapList : ListItem {
    array<Map@> maps;
    MapListType type = MapListType::None;
    string url;
    bool hascontents = false;
    bool open;

    MapList(const string &in name, const string &in identifier, const MapListType &in type, bool collapsed = false) {
        super(name, identifier);
        this.type = type;
        this.maps = {};
        this.hascontents = true; // shouldn't try to request using a url since it is empty
        this.open = !collapsed;
    }
    MapList(const string &in name, const string &in identifier, const MapListType &in type, array<Map@> maps, bool collapsed = false) {
        super(name, identifier);
        this.type = type;
        this.maps = maps;
        this.hascontents = true;
        this.open = !collapsed;
    }
    MapList(const string &in name, const string &in identifier, const MapListType &in type, const string &in url, bool collapsed = false) {
        super(name, identifier);
        this.type = type;
        this.url = url;
        this.hascontents = false;
        this.open = !collapsed;
    }
    
    void GetContents() {
        if (this.type == MapListType::Campaign) {
            NadeoActivities::GetMapsFromNadeoCampaign(NadeoServices::BaseURLLive() + "/api/token/club/" + this.url, this);
        } else if (this.type == MapListType::Room) {
            NadeoActivities::GetMapsFromNadeoRoom(NadeoServices::BaseURLLive() + "/api/token/club/" + this.url, this);
        } else if (this.type == MapListType::MapUploads) {
            NadeoActivities::GetMapsFromNadeoTrackUploads(NadeoServices::BaseURLLive() + "/api/token/club/" + this.url, this);
        }
    }

    // add a map to the maps list
    void InsertLast(Map@ map) {
        this.maps.InsertLast(map);
    }

    // remove all maps in the map list, keeping its name. Used for favourites
    void Clear() {
        this.maps = {};
    }

    // render the maps list (if its not collapsed)
    bool Render(const string &in index, int indent, bool allowclose = true) override {
        if (this.open && !this.hascontents) {
            startnew(CoroutineFunc(this.GetContents));
            this.hascontents = true; // set this now so it doesn't request again even if it doesn't show for a few frames
        }
        UI::SetCursorPos(UI::GetCursorPos() + vec2(indentWidth * indent,0));
        if (allowclose) {
            if (UI::Button(Icons::Times + '##:'+index)) {return true;}
        } else {
            UI::BeginDisabled();
            UI::Button(Icons::Times + '##:'+index);
            UI::EndDisabled();
        }
        UI::SameLine();
        if (this.open) {
            UI::SetNextItemOpen(this.open);
        }
        if (this.open = UI::CollapsingHeader(this.name + '##:'+index)) {
            if (this.maps.Length > 0) {
                for (uint i = 0; i < this.maps.Length; i++) {
                    if (this.maps[i].Render(index + "." + tostring(i), indent+1)) {
                        this.maps.RemoveAt(i);
                        i--;
                    }
                }
            }
        }
        return false;
    }

    bool Contains(Map@ map) {
        for (uint i = 0; i < this.maps.Length; i++) {
            if (this.maps[i] == map) {
                return true;
            }
        }
        return false;
    }
    bool Contains(string &in identifier) {
        for (uint i = 0; i < this.maps.Length; i++) {
            if (this.maps[i] == identifier) {
                return true;
            }
        }
        return false;
    }
}
