MapList@ viewedServerMaps = null;

namespace NadeoActivities {

    MapListType GetActivityType(const string &in type) {
        if (type == "campaign")
            return MapListType::Campaign;
        if (type == "room")
            return MapListType::Room;
        if (type == "skin-upload")
            return MapListType::SkinUploads;
        if (type == "map-upload")
            return MapListType::MapUploads;
        if (type == "map-review")
            return MapListType::MapReview;
        if (type == "news")
            return MapListType::News;
        if (type == "competition")
            return MapListType::Competition;
        if (type == "ranking-club")
            return MapListType::Rankings;
        if (type == "item-upload")
            return MapListType::ItemUploads;
        if (type == "ad")
            return MapListType::Advertisement;
        if (type == "team")
            return MapListType::Team;
        return MapListType::None;
    }

    // called when clicking on a room; the plugin will show the maps in a click-to-play format
    void _GetMapsFromNadeoRoom_viewed(const string &in activityurl) {
        // if getting maps for viewed server, check if its dedicated, (if so skip this, its detected differently)
        CTrackMania@ app = cast<CTrackMania@>(GetApp());
        if (app !is null &&
            app.ManiaTitleControlScriptAPI !is null &&
            app.ManiaTitleControlScriptAPI.GetServerInfo_Result !is null) {
                return;
        }
        @viewedServerMaps = MapList("", "", MapListType::Room, false);
        GetMapsFromNadeoRoom(activityurl, viewedServerMaps);
    }

    void GetMapsFromNadeoCampaign(const string &in campaignurl, MapList@ newmaps) {
        // request all the maps in the campaign
        while (!NadeoServices::IsAuthenticated("NadeoLiveServices")) {yield();}
        Net::HttpRequest@ req = NadeoServices::Get("NadeoLiveServices", campaignurl);
        req.Start();
        while(!req.Finished()) {yield();}
        if (req.ResponseCode() != 200) {
            print('invalid http response when getting campaign maps' + req.ResponseCode() + ', ' + req.String());
        }
        Json::Value@ response = Json::Parse(req.String());
        if (!response.HasKey('campaign') || !response['campaign'].HasKey('playlist')) {
            print('invalid response when getting campaign maps ' + req.String());
            return;
        }
        if (response['campaign']['playlist'].Length == 0) {
            print('campaign with no maps: ' + campaignurl);
            return;
        }
        // parse the map uids out of the response into a normal array
        array<string> responsemaps = {};
        for (uint i = 0; i < response['campaign']['playlist'].Length; i++) {
            responsemaps.InsertLast(response['campaign']['playlist'][i]['mapUid']);
        }
        if (newmaps.name == "") {
            newmaps.name = Text::OpenplanetFormatCodes(response['campaign']['name']);
        }
        if (newmaps.identifier == "") {
            newmaps.identifier = tostring(int(response['id']));
        }
        SetMapsFromUidList(responsemaps, newmaps);
    }

    void GetMapsFromNadeoRoom(const string &in activityurl, MapList@ newmaps) {
        // first re-request all the maps in the room
        while (!NadeoServices::IsAuthenticated("NadeoLiveServices")) {yield();}
        Net::HttpRequest@ req = NadeoServices::Get("NadeoLiveServices", activityurl);
        req.Start();
        while(!req.Finished()) {yield();}
        if (req.ResponseCode() != 200) {
            print('invalid http response when getting room maps' + req.ResponseCode() + ', ' + req.String());
        }
        Json::Value@ response = Json::Parse(req.String());
        if (!response.HasKey('room') || !response['room'].HasKey('maps')) {
            print('invalid response when getting room maps ' + req.String());
            return;
        }
        if (response['room']['maps'].Length == 0) {
            print('server with no maps: ' + activityurl);
            return;
        }
        // parse the map uids out of the response into a normal array
        array<string> responsemaps = {};
        for (uint i = 0; i < response['room']['maps'].Length; i++) {
            responsemaps.InsertLast(response['room']['maps'][i]);
        }
        if (newmaps.name == "") {
            newmaps.name = Text::OpenplanetFormatCodes(response['room']['name']);
        }
        if (newmaps.identifier == "") {
            newmaps.identifier = tostring(int(response['id']));
        }
        SetMapsFromUidList(responsemaps, newmaps);
    }

    void GetMapsFromNadeoTrackUploads(const string &in trackuploadsurl, MapList@ newmaps) {
        // request all the maps in the campaign
        int total = 1;
        int requested = 0;
        while (!NadeoServices::IsAuthenticated("NadeoLiveServices")) {yield();}
        array<string> responsemaps = {};
        Net::HttpRequest@ req;
        Json::Value@ response;
        while (total > requested) {
            @req = NadeoServices::Get("NadeoLiveServices", trackuploadsurl + "?length=100&offset="+requested);
            req.Start();
            while(!req.Finished()) {yield();}
            if (req.ResponseCode() != 200) {
                print('invalid http response when getting track upload maps' + req.ResponseCode() + ', ' + req.String());
            }
            @response = Json::Parse(req.String());
            if (!response.HasKey('bucketItemList')) {
                print('invalid response when getting track upload maps ' + req.String());
                return;
            }
            total = int(response['bucketItemCount']);
            if (response['bucketItemList'].Length == 0) {
                print('track upload with no maps: ' + trackuploadsurl);
                return;
            }
            // parse the map uids out of the response into a normal array
            for (uint i = 0; i < response['bucketItemList'].Length; i++) {
                responsemaps.InsertLast(response['bucketItemList'][i]['itemId']);
            }
            requested += 100;
        }
        if (newmaps.name == "") {
            newmaps.name = Text::OpenplanetFormatCodes(response['name']);
        }
        if (newmaps.identifier == "") {
            newmaps.identifier = tostring(int(response['id']));
        }
        SetMapsFromUidList(responsemaps, newmaps);
    }

    void SetMapsFromUidList(array<string>@ responsemaps, MapList@ newmaps) {
        // create a request to get the names of the maps
        string mapnamesurl = NadeoServices::BaseURLLive() + '/api/token/map/get-multiple?mapUidList=' + responsemaps[0];
        // nadeo has a limit of 100 maps per request
        for (uint i = 1; i < responsemaps.Length && i < 100; i++) { 
            mapnamesurl += ',' + responsemaps[i];
        }
        // request the info (for names) of the maps
        Net::HttpRequest@ req_names = NadeoServices::Get("NadeoLiveServices", mapnamesurl);
        req_names.Start();
        while(!req_names.Finished()) {yield();}
        if (req_names.ResponseCode() != 200) {
            print('invalid http response when getting activity maps map names' + req_names.ResponseCode() + ', ' + req_names.String());
        }
        Json::Value@ names_response = Json::Parse(req_names.String());
        if (!names_response.HasKey('mapList')) {
            print('invalid response when getting activity maps map names ' + req_names.String());
            return;
        }
        Json::Value@ infolist = names_response['mapList'];

        // I want to keep the order from the original request and don't know if the get-multiple is order consistent
        bool found;
        for (uint i = 0; i < responsemaps.Length; i++) {
            // order often seems consistent so check that first, but if not then check all
            if (i < infolist.Length && infolist[i]['uid'] == responsemaps[i]) {
                newmaps.InsertLast(Map(Text::OpenplanetFormatCodes(infolist[i]['name']), responsemaps[i], infolist[i]['mapType'], infolist[i]['downloadUrl']));
            } else {
                found = false;
                for (uint j = 0; j < infolist.Length; j++) {
                    if (j == i) {continue;} // this is pre-checked anyway (not like string comparison is expensive tho)
                    if (infolist[j]['uid'] == responsemaps[i]) {
                        newmaps.InsertLast(Map(Text::OpenplanetFormatCodes(infolist[j]['name']), responsemaps[i], infolist[j]['mapType'], infolist[j]['downloadUrl']));
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    // use uid as a backup if no name (map is probably invalid anyway unless there's over 100 maps)
                    // default type of "" only works for race maps
                    // default 'unkown url, look it up later'
                    print('no info response for map: ' + responsemaps[i]);
                    newmaps.InsertLast(Map(responsemaps[i], responsemaps[i], "", ""));
                }
            }
        }
    }

    void GetActivitiesFromClub(const string &in clubid) {
        // request all the activities in a club
        int total = 1;
        int requested = 0;
        string cluburl = NadeoServices::BaseURLLive() + "/api/token/club/" + clubid + "/activity";
        while (!NadeoServices::IsAuthenticated("NadeoLiveServices")) {yield();}
        Net::HttpRequest@ req;
        Json::Value@ response;
        // first request club name
        @req = NadeoServices::Get("NadeoLiveServices", NadeoServices::BaseURLLive() + "/api/token/club/" + clubid);
        req.Start();
        while(!req.Finished()) {yield();}
        if (req.ResponseCode() != 200) {
            print('invalid http response when getting club activities' + req.ResponseCode() + ', ' + req.String());
        }
        @response = Json::Parse(req.String());
        if (!response.HasKey('name')) {
            print('invalid response when getting club activities ' + req.String());
            return;
        }
        ListList@ activities = ListList(Text::OpenplanetFormatCodes(response['name']), clubid+'/activity', false);
        maplists.InsertFirst(activities);
        // then request club maps (in sets of 250)
        while (total > requested) {
            @req = NadeoServices::Get("NadeoLiveServices", cluburl + "?length=250&active=true&offset="+requested);
            req.Start();
            while(!req.Finished()) {yield();}
            if (req.ResponseCode() != 200) {
                print('invalid http response when getting club activities' + req.ResponseCode() + ', ' + req.String());
            }
            @response = Json::Parse(req.String());
            if (!response.HasKey('activityList')) {
                print('invalid response when getting club activities ' + req.String());
                return;
            }
            total = int(response['itemCount']);
            if (response['activityList'].Length == 0) {
                print('club with no activities: ' + clubid);
                return;
            }
            requested += 250;

            // parse the map uids out of the response into a normal array
            for (uint i = 0; i < response['activityList'].Length; i++) {
                MapListType type = GetActivityType(response['activityList'][i]['activityType']);
                string url = "";
                if (type == MapListType::Campaign) {
                    url = clubid + '/campaign/' + tostring(int(response['activityList'][i]['externalId'])); // for some reason campaign ids are different to activity ids
                    activities.InsertLast(MapList(Text::OpenplanetFormatCodes(response['activityList'][i]['name']), tostring(int(response['activityList'][i]['externalId'])), type, url, true));
                } else if (type == MapListType::Room) {
                    url = clubid + '/room/' + tostring(int(response['activityList'][i]['activityId']));
                    activities.InsertLast(MapList(Text::OpenplanetFormatCodes(response['activityList'][i]['name']), tostring(int(response['activityList'][i]['externalId'])), type, url, true));
                } else if (type == MapListType::MapUploads) {
                    url = clubid + '/bucket/' + tostring(int(response['activityList'][i]['activityId']));
                    activities.InsertLast(MapList(Text::OpenplanetFormatCodes(response['activityList'][i]['name']), tostring(int(response['activityList'][i]['activityId'])), type, url, true));
                } else if (type == MapListType::SkinUploads) {
                    url = clubid + '/bucket/' + tostring(int(response['activityList'][i]['activityId']));
                    activities.InsertLast(SkinList(Text::OpenplanetFormatCodes(response['activityList'][i]['name']), tostring(int(response['activityList'][i]['activityId'])), url, true));
                } else if (type == MapListType::News) {
                    url = clubid + '/news/' + tostring(int(response['activityList'][i]['activityId']));
                    activities.InsertLast(NewsList(Text::OpenplanetFormatCodes(response['activityList'][i]['name']), tostring(int(response['activityList'][i]['activityId'])), url, true));
                } else {
                    // maybe specific checks for other types?
                    activities.InsertLast(OtherActivity(Text::OpenplanetFormatCodes(response['activityList'][i]['name']), tostring(int(response['activityList'][i]['externalId']) == 0 ? int(response['activityList'][i]['activityId']) : int(response['activityList'][i]['externalId'])), true));
                }
            }
        }
    }

    void GetSkinsFromNadeoSkinUploads(const string &in skinuploadsurl, SkinList@ newskins) {
        // request all the maps in the campaign
        int total = 1;
        int requested = 0;
        while (!NadeoServices::IsAuthenticated("NadeoLiveServices")) {yield();}
        array<string> responseskins = {};
        Net::HttpRequest@ req;
        Json::Value@ response;
        while (total > requested) {
            @req = NadeoServices::Get("NadeoLiveServices", skinuploadsurl + "?length=100&offset="+requested);
            req.Start();
            while(!req.Finished()) {yield();}
            if (req.ResponseCode() != 200) {
                print('invalid http response when getting skin upload skins' + req.ResponseCode() + ', ' + req.String());
            }
            @response = Json::Parse(req.String());
            if (!response.HasKey('bucketItemList')) {
                print('invalid response when getting skin upload skins ' + req.String());
                return;
            }
            total = int(response['bucketItemCount']);
            if (response['bucketItemList'].Length == 0) {
                print('skin upload with no skins: ' + skinuploadsurl);
                return;
            }
            // parse the map uids out of the response into a normal array
            for (uint i = 0; i < response['bucketItemList'].Length; i++) {
                responseskins.InsertLast(response['bucketItemList'][i]['itemId']);
            }
            requested += 100;
        }
        if (newskins.name == "") {
            newskins.name = Text::OpenplanetFormatCodes(response['name']);
        }
        if (newskins.identifier == "") {
            newskins.identifier = tostring(int(response['externalId']));
        }
        // since there is no get skin info multi, each skin can do that itself
        for (uint i = 0; i < responseskins.Length; i++) {
            newskins.InsertLast(Skin("", responseskins[i], ""));
        }
    }

    void GetNewsFromNadeoNews(const string &in newsurl, NewsList@ newnews) {
        // request news activity text
        while (!NadeoServices::IsAuthenticated("NadeoLiveServices")) {yield();}
        Net::HttpRequest@ req;
        Json::Value@ response;
        @req = NadeoServices::Get("NadeoLiveServices", newsurl);
        req.Start();
        while(!req.Finished()) {yield();}
        if (req.ResponseCode() != 200) {
            print('invalid http response when getting news text' + req.ResponseCode() + ', ' + req.String());
        }
        @response = Json::Parse(req.String());
        if (!response.HasKey('body')) {
            print('invalid response when getting news text' + req.String());
            return;
        }
        newnews.header = Text::OpenplanetFormatCodes(response['headline']);
        newnews.body = Text::OpenplanetFormatCodes(response['body']);

    }
}

/*{
"creationTimestamp":1622297381,
"mediaUrlJpgLarge":"https://trackmania-prod-media-s3.cdn.ubi.com/media/image/live-api/be92e4d9-bc86-4a58-a7e7-8a6952f60ad3/jpg/large.jpg?timestamp=1705443859.jpg",
"mediaUrlJpgMedium":"https://trackmania-prod-media-s3.cdn.ubi.com/media/image/live-api/be92e4d9-bc86-4a58-a7e7-8a6952f60ad3/jpg/medium.jpg?timestamp=1705443859.jpg",
"clubName":"$s$03CA$15DL$17DT$29EE$2BER$3DFE$3FFD $03CN$16DA$29ED$2CEE$3FFO",
"id":153819,
"name":"Discord",
"clubId":24231,
"mediaUrl":"https://trackmania-prod-media-s3.cdn.ubi.com/media/image/live-api/be92e4d9-bc86-4a58-a7e7-8a6952f60ad3/dds/game.dds?timestamp=1705443859.dds",
"latestEditorAccountId":"300d6266-04ab-4f51-934a-f2837facb0f0",
"body":"Join our discord server to discuss the campaigns, make suggestions, submit new maps, or share your successes & fails!\n$03C$lhttps://discord.gg/8f3JcmT9tE$l\n\n$gEdge's \"Bad Ideas Zone\" club is the home of the Reverse and CPLess alterations. Please check it out and / or join their discord: $03C$l[https://discord.com/invite/uYh83uBghV]click here$l\n\n$g\"Flink's Club\" is the home of Icey alterations. Please check it out and / or join their discord: $03C$l[https://discord.com/invite/HvbxG9p]click here$l\n\n$g\"Nadeo 1UP\" & \"NADEO UNDERWATER\" are the homes of 1UP / Underwater alterations. Please check them out / or join their discord: $03C$l[https://discord.gg/66uJHvh6X6]click here$l",
"mediaTheme":"",
"creatorAccountId":"300d6266-04ab-4f51-934a-f2837facb0f0",
"headline":"Join the altered community!",
"mediaUrlJpgSmall":"https://trackmania-prod-media-s3.cdn.ubi.com/media/image/live-api/be92e4d9-bc86-4a58-a7e7-8a6952f60ad3/jpg/small.jpg?timestamp=1705443859.jpg",
"mediaUrlDds":"https://trackmania-prod-media-s3.cdn.ubi.com/media/image/live-api/be92e4d9-bc86-4a58-a7e7-8a6952f60ad3/dds/game.dds?timestamp=1705443859.dds"
}*/