
namespace TMXMappacks {
    // tmx short link of s/m/ or of mappack/ but without map name on the end
    bool LooksLikeShortLink(const string &in searchname) {
        if (Regex::IsMatch(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]s[\/\\]m[\/\\]\d{1,10}$""")) {
            return true;
        }
        return false;
    }
    // bool LooksLikeShortLink2(const string &in searchname) {
    //     if (Regex::IsMatch(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mappack[\/\\]\d{1,10}$""")) {
    //         return true;
    //     }
    //     return false;
    // }
    bool LooksLikeFullLink(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mappack[\/\\]\d{1,10}""")) {
            return true;
        }
        return false;
    }
    bool LooksLikeFullLink2(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mappack[\/\\]view[\/\\]\d{1,10}""")) {
            return true;
        }
        return false;
    }
    bool LooksLikeNewLink(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mappackshow[\/\\]\d{1,10}""")) {
            return true;
        }
        return false;
    }

    void SetMapsFromNumber(const string &in mappacknumber) {
        SetMaps(mappacknumber);
    }

    void SetMapsFromShortLink(const string &in mappacklink) {
        if (mappacklink.StartsWith('https:')) { // not checking for \\ or // because it can be either
            SetMapsFromNumber(mappacklink.SubStr(32));
        } else if (mappacklink.StartsWith('http:')) {
            SetMapsFromNumber(mappacklink.SubStr(31));
        } else {
            SetMapsFromNumber(mappacklink.SubStr(24));
        }
    }
    void SetMapsFromFullLink(string &in mappacklink) {
        if (mappacklink.StartsWith('https:')) {
            mappacklink = mappacklink.SubStr(36);
        } else if (mappacklink.StartsWith('http:')) {
            mappacklink = mappacklink.SubStr(35);
        } else {
            mappacklink = mappacklink.SubStr(28);
        }
        if (mappacklink.Contains("/")) {
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("/")));
        } else if (mappacklink.Contains("\\")) {
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("\\")));
        } else if (mappacklink.Length > 0) {
            SetMapsFromNumber(mappacklink);
        }
    }
    void SetMapsFromFullLink2(string &in mappacklink) {
        if (mappacklink.StartsWith('https:')) {
            mappacklink = mappacklink.SubStr(41);
        } else if (mappacklink.StartsWith('http:')) {
            mappacklink = mappacklink.SubStr(40);
        } else {
            mappacklink = mappacklink.SubStr(33);
        }
        if (mappacklink.Contains("/")) {
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("/")));
        } else if (mappacklink.Contains("\\")) {
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("\\")));
        } else if (mappacklink.Length > 0) {
            SetMapsFromNumber(mappacklink);
        }
    }
    void SetMapsFromNewLink(string &in mappacklink) {
        if (mappacklink.StartsWith('https:')) {
            mappacklink = mappacklink.SubStr(40);
        } else if (mappacklink.StartsWith('http:')) {
            mappacklink = mappacklink.SubStr(39);
        } else {
            mappacklink = mappacklink.SubStr(32);
        }
        if (mappacklink.Contains("/")) {
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("/")));
        } else if (mappacklink.Contains("\\")) {
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("\\")));
        } else if (mappacklink.Length > 0) {
            SetMapsFromNumber(mappacklink);
        }
    }

    void SetMaps(const string &in packid) {
        for (int i = maplists.Length - 1; i >= 0; i--) {
            if (maplists[i].identifier == "tmx:"+packid) {
                return;
            }
        }
        Net::HttpRequest@ req = Net::HttpGet('https://trackmania.exchange/api/mappacks?fields=Name,MapCount&count=1&id=' + packid);
        while(!req.Finished()) {yield();}
        if (req.ResponseCode() != 200 || req.String().Length == 0) {
            print('invalid http response when getting tmx mappack name' + req.ResponseCode() + ', ' + req.String());
            UI::ShowNotification("Play map extended", "Error getting mappack");
            return;
        }
        Json::Value@ response = Json::Parse(req.String());
        if (response["Results"].Length == 0) {
            print('no results when getting tmx mappack name' + response.GetType() + ', ' + req.String());
            UI::ShowNotification("Play map extended", "Couldn't find mappack");
            return;
        }
        string packname = packid;
        if (response["Results"][0]["Name"] != '') {
            packname = response["Results"][0]["Name"];
        }
        int count = int(response["Results"][0]["MapCount"]);
        @req = Net::HttpGet('https://trackmania.exchange/api/maps?fields=MapType,GbxMapName,MapId&count=' + count + '&mappackid=' + packid);
        while(!req.Finished()) {yield();}
        if (req.ResponseCode() != 200 || req.String().Length == 0) {
            print('invalid http response when getting mappack maps' + req.ResponseCode() + ', ' + req.String());
            UI::ShowNotification("Play map extended", "Error getting mappack maps");
            return;
        }
        @response = Json::Parse(req.String());
        if (response["Results"].Length == 0) {
            print('invalid response type when getting tmx mappack name' + response.GetType() + ', ' + req.String());
            UI::ShowNotification("Play map extended", "No maps found in mappack");
            return;
        }
        MapList@ newmaps = MapList(packname, "tmx:"+packid, MapListType::TMXMappack, false);
        for (uint i = 0; i < response["Results"].Length; i++) {
            newmaps.InsertLast(Map(Text::OpenplanetFormatCodes(response["Results"][i]["GbxMapName"]), tostring(int(response["Results"][i]["MapId"])), string(response["Results"][i]["MapType"]), 'https://trackmania.exchange/mapgbx/' + int(response["Results"][i]["MapId"]) + '?mappackId=' + packid));
        }
        if (bool(response["More"])) {
            print('tmx mappack unexpected More, ' + response["Results"].Length + ' maps out of expected ' + count +' given');
        }
        maplists.InsertFirst(newmaps);

    }
}
