
namespace TMXMaps {
    bool LooksLikeNumber(const string &in searchname) {
        if (Regex::IsMatch(searchname, """\d{1,10}""")) {
            return true;
        }
        return false;
    }
    // tmx short link of s/tr/ or of maps/ but without map name on the end
    bool LooksLikeShortLink(const string &in searchname) {
        if (Regex::IsMatch(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\](s[\/\\]tr|maps)[\/\\]\d{1,10}$""")) {
            return true;
        }
        return false;
    }
    bool LooksLikeFullLink(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]maps[\/\\]\d{1,10}""")) {
            return true;
        }
        return false;
    }
    bool LooksLikeNewLink(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mapshow[\/\\]\d{1,10}""")) {
            return true;
        }
        return false;
    }

    string GetDownloadFromNumber(const string &in mapnumber, string &out type, string &out mapname) {
        Net::HttpRequest@ req = Net::HttpGet('https://trackmania.exchange/api/maps?fields=MapType,GbxMapName&count=1&id=' + mapnumber);
        while (!req.Finished()) {
            yield();
        }
        if (req.String().Length == 0) {
            UI::ShowNotification("Play map extended", "Couldn't find map name");
            type = "";
            mapname = mapnumber;
        } else {
            Json::Value response = Json::Parse(req.String());
            if (response["Results"].Length == 0) {
                return "";
            }
            type = response["Results"][0]["MapType"];
            mapname = response["Results"][0]["GbxMapName"];
        }
        return 'https://trackmania.exchange/mapgbx/' + mapnumber;
    }

    string GetDownloadFromShortLink(const string &in maplink, string &out type, string &out mapname) {
        return GetDownloadFromNumber(GetNumberFromShortLink(maplink), type, mapname);
    }
    string GetNumberFromShortLink(const string &in maplink) {
        if (maplink.StartsWith('https:')) {
            return maplink.SubStr(33);
        } else if (maplink.StartsWith('http:')) {
            return maplink.SubStr(32);
        } else {
            return maplink.SubStr(25);
        }
    }
    string GetDownloadFromFullLink(const string &in maplink, string &out type, string &out mapname) {
        return GetDownloadFromNumber(GetNumberFromFullLink(maplink), type, mapname);
    }
    string GetNumberFromFullLink(string &in maplink) {
        if (maplink.StartsWith('https:')) {
            maplink = maplink.SubStr(33);
        } else if (maplink.StartsWith('http:')) {
            maplink = maplink.SubStr(32);
        } else {
            maplink = maplink.SubStr(25);
        }
        if (maplink.Contains("/")) {
            return maplink.SubStr(0, maplink.IndexOf("/"));
        } else if (maplink.Contains("\\")) {
            return maplink.SubStr(0, maplink.IndexOf("\\"));
        } else if (Text::ParseInt(maplink) != 0) {
            return maplink;
        }
        throw('cannot get number, invalid full link');
        return "";
    }
    string GetDownloadFromNewLink(const string &in maplink, string &out type, string &out mapname) {
        return GetDownloadFromNumber(GetNumberFromNewLink(maplink), type, mapname);
    }
    string GetNumberFromNewLink(string &in maplink) {
        if (maplink.StartsWith('https:')) {
            maplink = maplink.SubStr(36);
        } else if (maplink.StartsWith('http:')) {
            maplink = maplink.SubStr(35);
        } else {
            maplink = maplink.SubStr(28);
        }
        print(maplink);
        if (maplink.Contains("/")) {
            return maplink.SubStr(0, maplink.IndexOf("/"));
        } else if (maplink.Contains("\\")) {
            return maplink.SubStr(0, maplink.IndexOf("\\"));
        } else if (Text::ParseInt(maplink) != 0) {
            return maplink;
        }
        throw('cannot get number, invalid new link');
        return "";
    }

    string GetUidFromNumber(const string &in mapnumber) {
        Net::HttpRequest@ req = Net::HttpGet('https://trackmania.exchange/api/maps?fields=TrackUID&count=1&id=' + mapnumber);
        while (!req.Finished()) {
            yield();
        }
        if (req.String().Length == 0) {
            print('map name get');
            UI::ShowNotification("Play map extended", "Couldn't find map name");
            return "";
        } else {
            Json::Value response = Json::Parse(req.String());
            if (response["Results"].Length == 0) {
                return "";
            }
            return response["Results"][0]["TrackUID"];
        }
    }
    string GetUidFromShortLink(const string &in maplink) {
        return GetUidFromNumber(GetNumberFromShortLink(maplink));
    }
    string GetUidFromFullLink(string &in maplink) {
        return GetUidFromNumber(GetNumberFromFullLink(maplink));
    }
    string GetUidFromNewLink(string &in maplink) {
        return GetUidFromNumber(GetNumberFromNewLink(maplink));
    }

    string GetIdFromUid(const string &in uid) {
        Net::HttpRequest@ req = Net::HttpGet('https://trackmania.exchange/api/maps?fields=MapId&count=1&uid=' + uid);
        while (!req.Finished()) {
            yield();
        }
        if (req.ResponseCode() != 200) {
            print('invalid response from tmx for id from uid');
            return "-1";
        }
        if (req.String().Length == 0) {
            return "-1";
        }
        Json::Value response = Json::Parse(req.String());
        if (response['Results'].Length == 0) {
            // print('map not on tmx: ' + uid);
            return "-1";
        }
        return tostring(int(response['Results'][0]['MapId']));
        
    }
}
